/* avagl: avagl.cc
 * 
 * Copyright 2003 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <sstream>
#include <iomanip>
#include "parameters.hh"
#include "random.hh"
#include <gtk--.h>
#include "sandbox.hh"
#include "surface_profile.hh"

class AvaGUI
	: public Gtk::Window
{
private:
	Sandbox sb;
	SurfaceProfile sp;
	bool running;
	bool keep_running;
	guint32 delay;

	void prob_change(Gtk::Adjustment * a)
	{
		sb.set_prob(a->get_value());
	}

	int step;
	void step_change(Gtk::SpinButton * s)
	{
		step = s->get_value_as_int();
	}

	bool new_delay;
	void delay_change(Gtk::SpinButton * s)
	{
		delay = s->get_value_as_int();
		new_delay = true;
	}

	bool fresh;
	void toggle_fresh(Gtk::CheckButton * t)
	{
		fresh = t->get_active();
	}

	bool single;
	void toggle_single(Gtk::CheckButton * t)
	{
		single = t->get_active();
	}

	int done;

	SigC::Signal0<void> sig_update_count;
	void update_count(Gtk::Label * l)
	{
		std::ostringstream s;
		s << std::setw(8) << sb.get_count() << std::ends;
		l->set_text(s.str());
	}

	gint do_avalanche()
	{
		if (done < step) {
			if (! fresh) sb.iterate();
			done ++;
		} else {
			if (fresh) sb.start();
			sig_update_count.emit();
			// sp.rotate_y(1);
			sp.redraw();
			done = 0;
			if (single) stop_avalanche();
		}
		if (running) {
			if (new_delay) {
				if (delay == 0)
					Gtk::Main::idle.connect(slot(this, & AvaGUI::do_avalanche));
				else
					Gtk::Main::timeout.connect(slot(this, & AvaGUI::do_avalanche), delay);
				new_delay = false;
				return false;
			}
			return true;
		}
		return false;
	}

	void start_avalanche()
	{
		done = 0;
		if (!running) {
			running= true;
			if (delay == 0)
				Gtk::Main::idle.connect
					(slot(this, & AvaGUI::do_avalanche));
			else
				Gtk::Main::timeout.connect
					(slot(this, & AvaGUI::do_avalanche),
					 delay);
			new_delay = false;
		}
	}

	void stop_avalanche()
	{
		done = step;
		running = false;
	}

public:
	AvaGUI(unsigned w, unsigned l)
		: Gtk::Window(GTK_WINDOW_TOPLEVEL),
		  sp(w, l)
	{

		// initialize sandbox
		sb.initialize(w, l);

		sp.set_profile(sb.get_prof());
		sp.set_active(sb.get_active());
		sp.rotate_y(135);

		Gtk::Tooltips * tp = manage(new Gtk::Tooltips);
		Gtk::EventBox * eb;
		
		Gtk::Box * vb = manage(new Gtk::VBox(false, 0));
		add(* vb);

		// Upper Panel
		Gtk::Box * hb = manage(new Gtk::HBox(false, 0));
		vb->pack_start(* hb, true, true, 0);

		// Prob control
		Gtk::Adjustment * a = manage(new Gtk::Adjustment(0.5, 0, 1.0, 0.01));
		Gtk::VScale * v = manage(new Gtk::VScale(* a));
		eb = manage(new Gtk::EventBox);
		eb->add(* v);
		hb->pack_start(* eb, false, true, 0);
		v->set_digits(2);
		a->value_changed.connect(SigC::bind<Gtk::Adjustment *>(slot(this, & AvaGUI::prob_change), a));
		tp->set_tip(* eb, "propagation probability of avalanche activities");

		// SurfaceProfile Widget
		eb = manage(new Gtk::EventBox);
		eb->add(sp);
		hb->pack_start(* eb, true, true, 0);
		tp->set_tip(* eb, "drag to spin around the surface profile\n"
				"red area are sites participate in the last avalanche");
		// Upper Panel finish

		// Lower Panel (the Buttons)
		hb = manage(new Gtk::HBox(false, 0));
		vb->pack_start(* hb, false, false, 0);

		// Quit Button
		Gtk::Button * b = manage(new Gtk::Button("quit"));
		hb->pack_start(* b, false, false, 0);
		b->clicked.connect(destroy.slot());
		tp->set_tip(* b, "end avagl");

		// Avalanche Button
		b = manage(new Gtk::Button("aval"));
		hb->pack_start(* b, false, false, 0);
		b->clicked.connect(slot(this, & AvaGUI::start_avalanche));
		tp->set_tip(* b, "start avalanche");

		// Stop Button
		b = manage(new Gtk::Button("stop"));
		hb->pack_start(* b, false, false, 0);
		b->clicked.connect(slot(this, & AvaGUI::stop_avalanche));
		tp->set_tip(* b, "stop avalanche");

		// Refreshing Toggle
		Gtk::CheckButton * t = manage(new Gtk::CheckButton("fres"));
		hb->pack_start(* t, false, false, 0);
		t->set_active(false);
		fresh = false;
		t->toggled.connect
			(SigC::bind<Gtk::CheckButton *>
			 (slot(this, & AvaGUI::toggle_fresh), t));
		tp->set_tip(* t, "check to refresh the whole surface for each avalanche");

		// Refreshing Toggle
		t = manage(new Gtk::CheckButton("sing"));
		hb->pack_start(* t, false, false, 0);
		t->set_active(false);
		single = false;
		t->toggled.connect
			(SigC::bind<Gtk::CheckButton *>
			 (slot(this, & AvaGUI::toggle_single), t));
		tp->set_tip(* t, "check to do one avalanche at a time");

		// Avalanche Count Lable
		eb = manage(new Gtk::EventBox);
		Gtk::Label * lb = manage(new Gtk::Label("         0"));
		lb->set_usize(60, -1);
		eb->add(* lb);
		hb->pack_start(* eb, true, false, 0);
		sig_update_count.connect(SigC::bind<Gtk::Label *>
					 (slot(this, & AvaGUI::update_count), lb));
		tp->set_tip(* eb, "avalanche count");

		// Delay SpinButton and Adjustment
		a = manage(new Gtk::Adjustment(100, 0, 3000, 1, 10, 0));
		Gtk::SpinButton * s =
			manage(new Gtk::SpinButton(* a, 100, 0));
		hb->pack_start(* s, false, false, 0);
		a->value_changed.connect
			(SigC::bind<Gtk::SpinButton *>
			 (slot(this, & AvaGUI::delay_change), s));
		tp->set_tip(* s, "delay time (ms) between avalanches");

		// Step SpinButton and Adjustment
		a = manage(new Gtk::Adjustment(10, 1, 100000, 1, 20, 0));
		s = manage(new Gtk::SpinButton( * a, 1, 0));
		hb->pack_start(* s, false, false, 0);
		a->value_changed.connect(SigC::bind<Gtk::SpinButton *>(slot(this, & AvaGUI::step_change), s));
		tp->set_tip(* s, "number of avalanches before updating the surface profile");

		// Lower Panel finish
		// All finish
		tp->enable();
		destroy.connect(Gtk::Main::quit.slot());
		sb.start();
		done = 0;
		step = 10;
		running = false;
		keep_running = false;
		delay = 100;
		// Gtk::Main::idle.connect(slot(this, &AvaGUI::do_avalanche));
		show_all();
	}
	~AvaGUI() {}


};

int main(int argc, char *argv[], char *envp[])
{
	Parameters arg(argc, argv, envp);
	Gtk::Main m(&argc, &argv);
	AvaGUI ag(32, 64);
	ag.show();
	Gtk::Main::run();
	return 0;
}
